{
Object based handling of application and icons for PortableApps menu
handles multiple level categories and custom sorting.

Copyright 2007 Karl McMurdo

Portions Copyright 2006-2007 John T. Haller of PortableApps.com

Please see the included license file for the full details on
all included components.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


Todo: Adjust variable naming to match main portableappsmenu program style
      Add comments to all procedures/functions
      Different Icons for open and closed menus
      Rename Categories
      Custom dialog for exe (alternate hint and exit program/params)
      Custom Dialog for ctg (custom icons and Hint)
}
unit AppObjects;

interface

uses
  Windows, Classes, Graphics, Controls, Forms, Buttons, Menus, SysUtils, IniFiles, ShellAPI;

type
  TAppItem = class;
  TAppItemList = class;

  TAppSortType = ( astManual, astAlpha, astAlphaCtgFirst, astAlphaCtgLast);

  TAppItem = class(TObject)
  private
    fName:           String;
    fCaption:        String;
    fExePath:        String;
    fHint:           String;
    fIconPath:       String;
    fIconHandle:     HIcon;
    fExecOnClose:    Boolean;
    fAutoExec:       Boolean;
    fVisible:        Boolean;
    fSortOrder:      Integer;
    fExternalApp:    Boolean;
    fExternalPath:   String;
    // Menu (Category Parameters)
    fCategoryName:   String;
    fLevel:          Integer;
    fIsMenu:         Boolean;
    fMenuOpen:       Boolean;
    fOpenIconPath:   String;
    fOpenIconHandle: HIcon;
    // Exit Parameters
    fExitExe:        String; // App to run on menu exit IF app has run this session.
    fExitParams:     String; // Parameters for exit exe
    fExitOnlyIfRun:  Boolean; // Run ExitExe only if Application has Run this session
    fHasRun:         Boolean; // Indicates app has run since menu started
    fIsUpdating:     Boolean; // Indicates more than one option is being updated so don't update paf.
    fAppList:  TAppItemList; // So app can easily reference parent
    procedure SetCaption(strValue: String);
    procedure SetHint(strValue: String);
    procedure SetExecOnClose(bolValue: Boolean);
    procedure SetAutoExec(bolValue: Boolean);
    procedure SetVisible(bolValue: Boolean);
    procedure SetMenuOpen(bolValue: Boolean);
    procedure SetCategoryName(strValue: String);
    procedure SetExitExe(strValue: String);
    procedure SetExitParams(strValue: String);
    procedure SetIsUpdating(bolValue: Boolean);
    procedure SetSortOrder(intSortOrder: Integer);
    function GetSortString: String;
    constructor Create; overload;
    constructor Create(alParent: TAppItemList; var intItemIndex: Integer); overload;
    constructor CreateFromExe(strExePath: String; alParent: TAppItemList; var intItemIndex: Integer);
    constructor CreateFromDB(intDBID: Integer; DB: TIniFile; IconStream: TStream; alParent: TAppItemList; var intItemIndex: Integer);
    constructor CreateCategory(iniData: TIniFile; strSection: String; alParent: TAppItemList; var intItemIndex: Integer);
    constructor CreateNewCategory(strName: String; strCaption: String; strCategory: String;
      strIcoPath: String; strOpenIcoPath: String; alParent: TAppItemList; var intItemIndex: Integer);
    procedure SaveToDB(DBID: Integer; DB: TIniFile; IconStream: tStream);
    procedure SetModified; // Sets Modified flag in parent applist if applist is assigned
    procedure SetIconPath(strIcoPath: String);
    procedure SetOpenIconPath(strIcoPath: String);
    procedure UpdateSort;
  public
    property Name: String Read fName;
    property Caption: String Read fCaption Write SetCaption;
    property ExePath: String Read fExePath;
    property Hint: String Read fHint Write SetHint;
    property IconHandle: HIcon Read fIconHandle;
    property IconPath: String Read fIconPath Write SetIconPath;
    property OpenIconHandle: HIcon Read fOpenIconHandle;
    property OpenIconPath: String Read fOpenIconPath Write SetOpenIconPath;
    property ExecOnClose: Boolean Read fExecOnClose Write SetExecOnClose;
    property ExternalApp: Boolean Read fExternalApp;
    property AutoExec: Boolean Read fAutoExec Write SetAutoExec;
    property Visible: Boolean Read fVisible Write SetVisible;
    property SortOrder: Integer Read fSortOrder Write SetSortOrder;
    property Level: Integer Read fLevel;
    property CategoryName: String Read fCategoryName Write SetCategoryName;
    property IsMenu: Boolean Read fIsMenu;
    property MenuOpen: Boolean Read fMenuOpen Write SetMenuOpen;
    property HasRun: Boolean Read fHasRun Write fHasRun;
    property ExitExe: String Read fExitExe Write SetExitExe;
    property ExitParams: String Read fExitParams Write SetExitParams;
    property IsUpdating: Boolean Read fIsUpdating Write SetIsUpdating;
    property AppList: TAppItemList Read fAppList Write fAppList;
    function GetPAFFile: String;
    function PAFWriteable: Boolean;
    function AppExecute: Boolean;
    function ExitExecute: Boolean;
    function ItemIndex: Integer;
    function SortKey: String;
    procedure UpdatePAFData;
    destructor Destroy; override;
  end;

  TAppItemList = class(TObject)
  private
    fAppStringList:        TStringList;
    fDefaultIconPath:      String;  // Default Category Icon
    fDefaultIconHandle:    HIcon;  // Default Category Icon
    fDefaultOpenIconPath:  String;  // Default Category Open Icon
    fDefaultOpenIconHandle: HIcon;  // Default Category Open Icon
    fReadOnly:         Boolean; // Main app must set this if drive isn't writeable
    fModified:         Boolean;
    fShowAll:          Boolean;
    fMenuIndent:       Integer; // Amount to Indent Each SubMenu.
    fDataFile:         String;  // Main Application INI File Name (Defaults to App.ini)
    fAppDBname:        String;  // DB Storage File Name (Defaults to App.fsa and Apps.fsi)
    fExtAppDir:        String;  // External App Data Directory
    fOnChanged:        TNotifyEvent;
    fBeforeAppExecute: TNotifyEvent;
    fAfterAppExecute:  TNotifyEvent;
    fUpdateCount:      Integer;
    fTransColor:       TColor;
    fAppFont:          TFont;
    fMenuFont:         TFont;
    fMenuOpenFont:     TFont;
    fLastSort:         TAppSortType;
    fBasePath:         String; // Fully Qualified path of portableappsmenu
    function GetAppItem(Index: Integer): TAppItem;
    procedure SetDefaultIcon(strValue: String);
    procedure SetDefaultOpenIcon(strValue: String);
    procedure SetModified(bolValue: Boolean);
    procedure SetUpdateCount(intValue: Integer);
    procedure SetSortString(intItemIndex: Integer; strValue: String);
  public
    ButtonsArray: array of TSpeedButton;
    property OnChanged: TNotifyEvent Read fOnChanged Write fOnChanged;
    property BeforeAppExecute: TNotifyEvent Read fBeforeAppExecute Write fBeforeAppExecute;
    property AfterAppExecute: TNotifyEvent Read fAfterAppExecute Write fAfterAppExecute;
    property Apps[Index: Integer]: TAppItem Read GetAppItem; Default;
    property DefaultIconPath: String Read fDefaultIconPath Write SetDefaultIcon;
    property DefaultOpenIconPath: String Read fDefaultOpenIconPath Write SetDefaultOpenIcon;
    property ReadOnly: Boolean Read fReadOnly Write fReadOnly;
    property Modified: Boolean Read fModified Write SetModified;
    property ShowAll: Boolean Read fShowAll Write fShowAll;
    property DataFile: String Read fDataFile Write fDataFile;
    property AppDBName: String Read fAppDBName Write fAppDBName;
    property UpdateCount: Integer Read fUPdateCount Write SetUpdateCount;
    property TransColor: TColor Read fTransColor Write fTransColor;
    property AppFont: TFont Read fAppFont Write fAppFont;
    property MenuFont: TFont Read fMenuFont Write fMenuFont;
    property MenuOpenFont: TFont Read fMenuOpenFont Write fMenuOpenFont;
    property MenuIndent: Integer Read fMenuIndent Write fMenuIndent;
    property LastSort: TAppSortType Read fLastSort Write fLastSort;
    function Count: Integer;
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Delete(ItemIndex: Integer);
    function IsVisible(ItemIndex: Integer): Boolean;
    function FirstVisible: Integer;
    function PrevVisible(From: Integer): Integer;
    function NextVisible(From: Integer): Integer;
    function LastVisible: Integer;
    function AtTop: Boolean;
    function AtBottom: Boolean;
    function AddApp(ExePath: String): Integer; overload;
    function AddApp(DBID: Integer; DB: TIniFile; IconStream: TStream): Integer; overload;
    function AddExternalApp(strExePath: String): Integer;
    function AddCategory(strName: String; strCaption: String; strCategory: String;
      bolIsOpen: Boolean; intSortOrder: Integer; strIcoPath: String;
      strOpenIcoPath: String): Integer;
    procedure DelCategory(strName: String); overload;
    procedure DelCategory(ItemIndex: Integer); overload;
    function GetCtgByName(CtgName: String): Integer;
    procedure ToggleMenu(ItemIndex: Integer);
    procedure SaveToDB; overload;
    function LoadFromDB: Boolean; overload;
    procedure SaveToDB(DBFileName: String); overload;
    function LoadFromDB(DBFileName: String): Boolean; overload;
    function LoadFromPath(strPortableAppsPath: String; Excludes: TStrings): Boolean;
    function LoadCategories: Integer;
    procedure SaveCategories;
    procedure CategoryMenu(ItemIndex: Integer; mnuParent: TMenuItem);
    procedure mnuCategoryClick(Sender: TObject);
    procedure Apply(intIndex: Integer; intTarget: Integer);
    procedure DrawButtons;
    procedure ForceVisible(ItemIndex: Integer);
    procedure ShowSubItems(ItemIndex: Integer);
    procedure FixLevels; // Really bad hack, should traverse tree rather then doing all
    procedure ReSort; overload;
    procedure ReSort(astSortType: TAppSortType); overload;
    function CanSortUp(ItemIndex: Integer): Boolean;
    function CanSortDown(ItemIndex: Integer): Boolean;
    function SortUp(ItemIndex: Integer): Integer;
    function SortDown(ItemIndex: Integer): Integer;
    function SortTop(ItemIndex: Integer): Integer;
    function SortBottom(ItemIndex: Integer): Integer;
    function PrepareSort(ItemIndex: Integer; slWork: TStringList): Integer;
    procedure FinalizeSort(slWork: TStringList);
    procedure Scroll(intSteps: Integer);
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure AutoExec;
    function AppExecute(ItemIndex: Integer): Boolean;
    procedure SetFontSize(intSize: Integer);
    procedure ExitExec;
    procedure ToggleAll(bolOpen: Boolean);
    function FullPath(strPath: String): String;
    function RelativePath(strPath: String): String;
    procedure CleanupSortOrder_UNUSED;
  end;

const
  AppDBVersion = '2.0';


implementation

function GetIconFromFile(Path: String): HIcon;
// Load the icon from a file (if it exists)
var
  FileInfo: TSHFileInfo;
begin
  Result := 0;
  if FileExists(Path) then
  begin
    FillChar(FileInfo, SizeOf(FileInfo), #0);
    if SHGetFileInfo(PChar(Path), 0, FileInfo, SizeOf(FileInfo), SHGFI_ICON or
        SHGFI_SMALLICON) <> 0 then
      Result := FileInfo.hIcon;
  end;
end;

constructor TAppItem.Create;
// Base create of an AppItem clearing properties
begin
  inherited;
  fName           := '';
  fCaption        := '';
  fExePath        := '';
  fHint           := '';
  fIconPath       := '';
  fIconHandle     := 0;
  fOpenIconHandle := 0;
  fExecOnClose    := False;
  fExternalApp    := False;
  fExternalPath   := '';
  fAutoExec       := False;
  fVisible        := True; // is Application Hidden
  fSortOrder      := 0;
  fLevel          := 0;
  fCategoryName   := '';
  fIsMenu         := False;
  fMenuOpen       := False;
  fExitExe        := '';
  fExitParams     := '';
  fHasRun         := False;
  fIsUpdating     := False;
end;

constructor TAppItem.Create(alParent: TAppItemList; var intItemIndex: Integer);
// Create TAppItem Linked to Parent TAppItemList
begin
  Create;
  fAppList := alParent;
  intItemIndex := fAppList.fAppStringList.AddObject('', self);
end;

constructor TAppItem.CreateFromDB(intDBID: Integer; DB: TIniFile; IconStream: TStream; alParent: TAppItemList; var intItemIndex: Integer);
// Create and AppItem from the saved database
var
  Section: String;
  Istr:    TMemoryStream;
  Isz:     Integer;
  Ico:     TIcon;
begin
  Create(alParent, intItemIndex);
  Section       := format('APP-%3.3d', [intDBID]);
  fName         := DB.ReadString(Section, 'Name', '');
  fCaption      := DB.ReadString(Section, 'Caption', fName);
  fExePath      := fAppList.FullPath(DB.ReadString(Section, 'ExePath', ''));
  fHint         := DB.ReadString(Section, 'Hint', 'Location: ' + ExtractFileDir(fExePath));
  fExecOnClose  := DB.ReadBool(Section, 'ExecOnClose', False);
  fAutoExec     := DB.ReadBool(Section, 'AutoExec', False);
  fExternalApp  := DB.ReadBool(Section, 'ExternalApp', False);
  fExternalPath := fAppList.FullPath(DB.ReadString(Section, 'ExternalPath', ''));
  fVisible      := DB.ReadBool(Section, 'Visible', True);
  fSortOrder    := DB.ReadInteger(Section, 'SortOrder', 0);
  fLevel        := DB.ReadInteger(Section, 'Level', 0);
  fCategoryName := DB.ReadString(Section, 'CategoryName', '');
  fIsMenu       := DB.ReadBool(Section, 'IsMenu', False);
  fMenuOpen     := DB.ReadBool(Section, 'MenuOpen', False);
  fExitExe      := fAppList.FullPath(DB.ReadString(Section, 'ExitExe', ''));
  fExitParams   := DB.ReadString(Section, 'ExitParams', '');
  fIsUpdating   := False;
  fHasRun       := False;
  fIconHandle := 0;
  fOpenIconHandle := 0;
  Isz := DB.ReadInteger(Section, 'IcoSize', 0);
  if (Isz > 0) then
  begin
    // Load Icon Path only if there was an icon
    fIconPath := fAppList.FullPath(DB.ReadString(Section, 'IconPath', ''));
    Ico  := TIcon.Create;
    IStr := TMemoryStream.Create;
    Istr.CopyFrom(IconStream, Isz);
    Istr.Seek(0, soFromBeginning);
    Ico.LoadFromStream(Istr);
    Istr.Clear;
    Istr.Free;
    fIconHandle := CopyIcon(Ico.Handle);
    Ico.Free;
  end;
  if fIsMenu then
  begin
    Isz := DB.ReadInteger(Section, 'OpenIcoSize', 0);
    if (Isz > 0) then
    begin
      // Load Open Icon Path only if there was an open icon
      fOpenIconPath := fAppList.FullPath(DB.ReadString(Section, 'OpenIconPath', ''));
      Ico  := TIcon.Create;
      IStr := TMemoryStream.Create;
      Istr.CopyFrom(IconStream, Isz);
      Istr.Seek(0, soFromBeginning);
      Ico.LoadFromStream(Istr);
      Istr.Clear;
      Istr.Free;
      fOpenIconHandle := CopyIcon(Ico.Handle);
      Ico.Free;
    end;
  end;
  fAppList.SetSortString(intItemIndex, GetSortString);
  intItemIndex := ItemIndex;
end;

constructor TAppItem.CreateFromExe(strExePath: String; alParent: TAppItemList; var intItemIndex: Integer);
// Create an Application from an Exe Path
var
  ptrFileInfo: pointer;
  lngFileInfoSize: Longint;
  dwdFileInfoLength: DWORD;
  ptrFileInfoLanguage: Pointer;
  ptrFileInfoData: Pointer;
  strFileInfoQuery: String;
  uintFileInfoDataLength: Uint;
  intCtgIndex: Integer;
begin
  Create(alParent, intItemIndex);
  if AnsiSameText(ExtractFileExt(strExePath), '.PAFDATA') then
  begin
    if FileExists(strExePath) then
    with TIniFile.Create(strExePath) do
    begin
      fExternalPath := strExePath;
      strFileInfoQuery := ReadString('PAFDATA', 'ExePath', '');
      strExePath := fAppList.FullPath(strFileInfoQuery);
      if AnsiSameText(strExePath, strFileInfoQuery) then
        WriteString('PAFDATA', 'ExePath', fAppList.RelativePath(strExePath));
      fExternalApp := True;
    end;
  end;
  if FileExists(strExePath) then
  begin
    fExePath := strExePath;
    lngFileInfoSize := GetFileVersionInfoSize(PChar(fExePath), dwdFileInfoLength);
    if (lngFileInfoSize > 0) then
    begin
      GetMem(ptrFileInfo, lngFileInfoSize);
      try
        // Check if the current app has a ProductName
        if GetFileVersionInfo(PChar(fExePath), dwdFileInfoLength, lngFileInfoSize,
          ptrFileInfo) then
        begin
          strFileInfoQuery := 'ProductName';
          if VerQueryValue(ptrFileInfo, '\VarFileInfo\Translation',
            ptrFileInfoLanguage, uintFileInfoDataLength) then
          begin
            strFileInfoQuery := Format('\StringFileInfo\%0.4x%0.4x\%s'#0,
              [LoWord(Longint(ptrFileInfoLanguage^)), HiWord(Longint(ptrFileInfoLanguage^)),
              strFileInfoQuery]);
          end;
          if VerQueryValue(ptrFileInfo, PChar(strFileInfoQuery),
            ptrFileInfoData, uintFileInfoDataLength) then
          begin
            fName := strPas(ptrFileInfoData);
          end;
        end;
      finally
        FreeMem(ptrFileInfo, lngFileInfoSize);
      end;
      GetMem(ptrFileInfo, lngFileInfoSize);
      if (trim(fName) = '') then
      begin
        try
          // Check if the current app has a FileDescription
          if GetFileVersionInfo(PChar(fExePath), dwdFileInfoLength,
            lngFileInfoSize, ptrFileInfo) then
          begin
            strFileInfoQuery := 'FileDescription';
            if VerQueryValue(ptrFileInfo, '\VarFileInfo\Translation',
              ptrFileInfoLanguage, uintFileInfoDataLength) then
            begin
              strFileInfoQuery := Format('\StringFileInfo\%0.4x%0.4x\%s'#0,
                [LoWord(Longint(ptrFileInfoLanguage^)), HiWord(Longint(ptrFileInfoLanguage^)),
                strFileInfoQuery]);
            end;
            if VerQueryValue(ptrFileInfo, PChar(strFileInfoQuery),
              ptrFileInfoData, uintFileInfoDataLength) then
            begin
              fName := strPas(ptrFileInfoData);
            end;
          end;
        finally
          FreeMem(ptrFileInfo, lngFileInfoSize);
        end;
      end;
    end;
    if (trim(fName) = '') then
    begin
      fName := ChangeFileExt(ExtractFileName(fExePath), '');
    end; // Default Name
    if FileExists(GetPAFFile) then
    begin
      with TIniFile.Create(GetPAFFile) do
      begin
        fCaption      := ReadString('PAFDATA', 'Name', fName);
        fVisible      := StrToBoolDef(ReadString('PAFDATA', 'Visible', 'True'), True);
        fAutoExec     := StrToBoolDef(ReadString('PAFDATA', 'AutoExec', 'False'), False);
        fHint         := ReadString('PAFDATA', 'Hint', fHint);
        fExecOnClose  := StrToBoolDef(ReadString('PAFDATA', 'ExecOnClose', 'False'), False);
        fCategoryName := ReadString('PAFDATA', 'Category', '');
        fIconPath     := ReadString('PAFDATA', 'Icon', '');
        fExitExe      := ReadString('PAFDATA', 'ExitExe', '');
        fExitParams   := ReadString('PAFDATA', 'ExitParams', '');
        Free;
      end;
      if ( fCategoryName <> '' ) then
      begin
        intCtgIndex := fAppList.GetCtgByName(fCategoryName);
        if intCtgIndex >= 0 then
          fLevel := fAppList.Apps[intCtgIndex].Level + 1;
      end;
    end;
    with TIniFile.Create(fAppList.DataFile) do
    begin
      fSortOrder := ReadInteger('SortOrder', SortKey, 0);
      Free;
    end;
    if (fCaption = '') then
    begin
      fCaption := fName;
    end;
    if (fHint = '') then
    begin
      fHint := 'Location: ' + ExtractFileDir(fExePath);
    end;
    fIconHandle := GetIconFromFile(fIconPath);
    if fIconHandle = 0 then // Icon mustn't exist
    begin
      fIconPath := fExePath;
      fIconHandle := GetIconFromFile(fIconPath);
    end;
    fAppList.SetSortString(intItemIndex, GetSortString);
    intItemIndex := ItemIndex;
  end;
end;

constructor TAppItem.CreateNewCategory(strName: String; strCaption: String; strCategory: String;
      strIcoPath: String; strOpenIcoPath: String; alParent: TAppItemList; var intItemIndex: Integer);
// Create a new category
var
  intCtgIndex: Integer;
begin
  Create(alParent, intItemIndex);
  fIsMenu := True;
  fName := strName;
  fCaption := strCaption;
  fCategoryName := strCategory;
  if fCategoryName <> '' then
  begin
    intCtgIndex := fAppList.GetCtgByName(fCategoryName);
    if intCtgIndex >= 0 then
      fLevel := fAppList.Apps[intCtgIndex].Level + 1;
  end;
  if ( strIcoPath <> '' ) and ( not AnsiSameText(strIcoPath, fAppList.fDefaultIconPath ) ) then
  begin
    fIconHandle := GetIconFromFile(strIcoPath);
    if fIconHandle <> 0 then
      fIconPath := strIcoPath;
  end;
  if ( strOpenIcoPath <> '' ) and ( not AnsiSameText(strOpenIcoPath, fAppList.fDefaultOpenIconPath ) ) then
  begin
    fOpenIconHandle := GetIconFromFile(strOpenIcoPath);
    if fOpenIconHandle <> 0  then
      fOpenIconPath := strOpenIcoPath; // add open icon (if any)
  end;
  with TIniFile.Create(fAppList.DataFile) do
  begin
    fSortOrder := ReadInteger('SortOrder', SortKey, 0);
    Free;
  end;
  fAppList.SetSortString(intItemIndex, GetSortString);
  intItemIndex := ItemIndex;
end;

constructor TAppItem.CreateCategory(iniData: TIniFile; strSection: String; alParent: TAppItemList; var intItemIndex: Integer);
// Load a Saved Category
var
  intCtgIndex: Integer;
begin
  Create(alParent, intItemIndex);
  fIsMenu := True;
  fName := iniData.ReadString(strSection, 'Name', strSection);
  fCaption  := iniData.ReadString(strSection, 'Caption', strSection);
  fCategoryName := iniData.ReadString(strSection, 'Category', '');
  if fCategoryName <> '' then
  begin
    intCtgIndex := fAppList.GetCtgByName(fCategoryName);
    if intCtgIndex >= 0 then
      fLevel := fAppList.Apps[intCtgIndex].Level + 1;
  end;
  fIconPath := iniData.ReadString(strSection, 'Icon', '');
  if fIconPath <> '' then
    fIconPath := fAppList.FullPath(fIconPath);
  if ( fIconPath <> '' ) and ( not AnsiSameText(fIconPath, fAppList.fDefaultIconPath ) ) then
  begin
    fIconHandle := GetIconFromFile(fIconPath);
    if fIconHandle = 0 then
      fIconPath := '';
  end;
  fOpenIconPath := iniData.ReadString(strSection, 'OpenIcon', '');
  if fOpenIconPath <> '' then
    fOpenIconPath := fAppList.FullPath(fOpenIconPath);
  if ( fOpenIconPath <> '' ) and ( not AnsiSameText(fOpenIconPath, fAppList.fDefaultOpenIconPath ) ) then
  begin
    fOpenIconHandle := GetIconFromFile(fOpenIconPath);
    if fOpenIconHandle = 0  then
      fOpenIconPath := ''; // add open icon (if any)
  end;
  fSortOrder := iniData.ReadInteger('SortOrder', SortKey, 0);
  fAppList.SetSortString(intItemIndex, GetSortString);
  intItemIndex := ItemIndex;
end;

destructor TAppItem.Destroy;
begin
  SetModified;
  if (fIconHandle <> 0) then
  begin
    DestroyIcon(fIconHandle);
  end;
  if (fOpenIconHandle <> 0) then
  begin
    DestroyIcon(fOpenIconHandle);
  end;
  inherited Destroy;
end;

procedure TAppItem.SetIconPath(strIcoPath: String);
begin
  if strIcoPath <> fIconPath then
  begin
    if fIconHandle <> 0 then
    begin
      DestroyIcon(fIconHandle);
      fIconHandle := 0;
    end;
    fIconPath := '';
    if Assigned(fAppList) then
      if AnsiSameText(strIcoPath, fAppList.DefaultIconPath) then
        strIcoPath := '';
    if strIcoPath <> '' then
    begin
      fIconHandle := GetIconFromFile(strIcoPath);
      if fIconHandle <> 0 then
        fIconPath := strIcoPath;
    end;
  end;
end;

procedure TAppItem.SetOpenIconPath(strIcoPath: String);
// Set the open icon for a menu item
begin
  if not fIsMenu then
    Exit; // Can only set this on a menu.
  if not AnsiSameText(strIcoPath, fOpenIconPath) then
  begin
    if (fOpenIconHandle <> 0) then
    // Destroy old icon
    begin
      DestroyIcon(fOpenIconHandle);
      fOpenIconHandle := 0;
    end;
    fOpenIconPath := '';
    if Assigned(fAppList) then
      if AnsiSameText(strIcoPath, fAppList.DefaultOpenIconPath) then
        strIcoPath := '';
    if strIcoPath <> '' then
    begin
      fOpenIconHandle := GetIconFromFile(strIcoPath);
      if fOpenIconHandle <> 0 then
        fOpenIconPath := strIcoPath;
    end;
    SetModified;
  end;
end;

procedure TAppItem.SetModified;
begin
  if Assigned(fAppList) then
  begin
    fAppList.Modified := True;
  end;
end;

procedure TAppItem.UpdateSort;
begin
  if Assigned(fAppList) then
  begin
    fAppList.fAppStringList.Sorted     := False;
    fAppList.fAppStringList[ItemIndex] := GetSortString;
    fAppList.fAppStringList.Sort;
    fAppList.fAppStringList.Sorted     := True;
    fAppList.FixLevels;
  end;
end;

procedure TAppItem.SetCaption(strValue: String);
begin
  if strValue <> fCaption then
  begin
    fCaption := strValue;
    UpdatePAFData;
    UpdateSort;
    SetModified;
  end;
end;

procedure TAppItem.SetHint(strValue: String);
begin
  if strValue <> fHint then
  begin
    fHint := strValue;
    UpdatePAFData;
    SetModified;
  end;
end;

procedure TAppItem.SetExecOnClose(bolValue: Boolean);
begin
  if bolValue <> fExecOnClose then
  begin
    fExecOnClose := bolValue;
    UpdatePAFData;
    SetModified;
  end;
end;

procedure TAppItem.SetAutoExec(bolValue: Boolean);
begin
  if bolValue <> fAutoExec then
  begin
    fAutoExec := bolValue;
    UpdatePAFData;
    SetModified;
  end;
end;

procedure TAppItem.SetVisible(bolValue: Boolean);
begin
  if bolValue <> fVisible then
  begin
    fVisible := bolValue;
    UpdatePAFData;
    SetModified;
  end;
end;

procedure TAppItem.SetMenuOpen(bolValue: Boolean);
begin
  if bolValue <> fMenuOpen and fIsMenu then
  begin
    fAppList.BeginUpdate;
    fMenuOpen := bolValue;
    fAppList.EndUpdate;
  end;
end;

procedure TAppItem.SetCategoryName(strValue: String);
begin
  if not AnsiSameText(strValue, fCategoryName) then
  begin
    fCategoryName := strValue;
    UpdatePAFData;
    UpdateSort;
    if Assigned(fAppList) then
    begin
      TAppItemList(fAppList).FixLevels;
    end;
    SetModified;
  end;
end;

procedure TAppItem.SetSortOrder(intSortOrder: Integer);
begin
  if intSortOrder <> fSortOrder then
  begin
    fSortOrder := intSortOrder;
    with TIniFile.Create(fAppList.DataFile) do
    begin
      if fSortOrder = 0 then
        DeleteKey('SortOrder', SortKey)
      else
        WriteInteger('SortOrder', SortKey, fSortOrder);
      Free;
    end;
    fAppList.LastSort := astManual;
    SetModified;
  end;
end;

procedure TAppItem.SetExitExe(strValue: String);
begin
  if not AnsiSameText(strValue, fExitExe) then
  begin
    fExitExe := strValue;
    UpdatePAFData;
    SetModified;
  end;
end;

procedure TAppItem.SetExitParams(strValue: String);
begin
  if strValue <> fExitParams then
  begin
    fExitParams := strValue;
    UpdatePAFData;
    SetModified;
  end;
end;

procedure TAppItem.SetIsUpdating(bolValue: Boolean);
begin
  if bolValue <> fIsUpdating then
  begin
    fIsUpdating := bolValue;
    if not fIsUpdating then
    begin
      UpdatePAFData;
    end;
  end;
end;

procedure TAppItem.UpdatePAFData;
var
  bolDefault: Boolean;  // Track if any of the items are non default
begin
  if fIsUpdating then
  begin
    Exit;
  end; // Multiple Changes so wait till IsUpdating is Unset
  if fIsMenu then
  begin
        //  if Assigned(fAppList) then
        //    TAppItemList(fAppList).SaveCategories;
  end else
  begin // Standard App
    if FileExists(GetPAFFile) and FileIsReadOnly(GetPAFFile) then
    begin
      Exit;
    end; // File is readonly
    if FileIsReadOnly(ExtractFileDir(GetPAFFile)) then
    begin
      Exit;
    end; // Directory is read only
    if (Assigned(fAppList)) then
    begin
      if TAppItemList(fAppList).ReadOnly then
      begin
        Exit;
      end;
    end; // Main app has specified read only
    bolDefault := True;
    with TIniFile.Create(GetPAFFile) do
    begin
      try
        if (fCaption <> fName) then
        begin
          WriteString('PAFDATA', 'Name', fCaption);
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'Name');
        end;
        if (not fVisible) then
        begin
          WriteString('PAFDATA', 'Visible', BoolToStr(fVisible, True));
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'Visible');
        end;
        if (fAutoExec) then
        begin
          WriteString('PAFDATA', 'AutoExec', BoolToStr(fAutoExec, True));
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'AutoExec');
        end;
        if (not AnsiSameText(fHint, 'Location: ' + ExtractFileDir(fExePath))) then
        begin
          WriteString('PAFDATA', 'Hint', fHint);
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'Hint');
        end;
        if (fExecOnClose) then
        begin
          WriteString('PAFDATA', 'ExecOnClose', BoolToStr(fExecOnClose, True));
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'ExecOnClose');
        end;
        if (fCategoryName <> '') then
        begin
          WriteString('PAFDATA', 'Category', fCategoryName);
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'Category');
        end;
        if (fExitExe <> '') then
        begin
          WriteString('PAFDATA', 'ExitExe', fAppList.RelativePath(fExitExe));
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'ExitExe');
        end;
        if (fExitParams <> '') then
        begin
          WriteString('PAFDATA', 'ExitParams', fExitParams);
          bolDefault := False;
        end else
        begin
          DeleteKey('PAFDATA', 'ExitParams');
        end;
      finally
        Free;
      end;
    end;
    if FileExists(GetPAFFile) and bolDefault then
    begin
      DeleteFile(GetPAFFile);
    end; // All Values are at default so kill PAF file
  end;
end;

function TAppItem.AppExecute: Boolean;
var
  strDir: String;
begin
  Result := False;
  if Assigned(fAppList) then
  begin
    if Assigned(TAppItemList(fAppList).BeforeAppExecute) then
    begin
      TAppItemList(fAppList).BeforeAppExecute(Self);
    end;
  end;
  if isMenu then
  begin
    if Assigned(fAppList) then
    begin
      TAppItemList(fAppList).ToggleMenu(ItemIndex);
    end;
  end else if FileExists(fExePath) then
  begin
    strDir  := ExtractFilePath(fExePath);
    fHasRun := (ShellExecute(0, 'open', PAnsiChar(fExePath), nil, PAnsiChar(strDir),
      SW_SHOW) > 32);
    Result  := fHasRun;
  end;
  if Assigned(fAppList) then
  begin
    if Assigned(TAppItemList(fAppList).AfterAppExecute) then
    begin
      TAppItemList(fAppList).AfterAppExecute(Self);
    end;
  end;
end;

function TAppItem.ExitExecute: Boolean;
var
  strExe: String;
  strDir: String;
  strPrm: String;
begin
  Result := False;
  strExe := fExePath;
  strPrm := fExitParams;
  if fExitExe <> '' then
  begin
    // if ExeExe is set the only do exitexec if app has run
    if fHasRun then
    begin
      strExe := fExitExe;
    end
    else
    begin
      strExe := '';
    end;
  end;
  if strExe <> '' then
  begin
    if FileExists(strExe) then
    begin
      strDir := ExtractFilePath(strExe);
      ShellExecute(0, 'open', PAnsiChar(strExe), PAnsiChar(strPrm), PAnsiChar(strDir), SW_SHOW);
      Result := fHasRun;
    end;
  end;
end;

function TAppItem.GetSortString: String;
var
  strWork: String;
  intCtg:  Integer;
begin
  strWork := Format('%3.3d %-20.20s', [fSortOrder, fCaption]);
  if (fCategoryName <> '') then
  begin
    if Assigned(fAppList) then
    begin
      with TAppItemList(fAppList) do
      begin
        intCtg := GetCtgByName(fCategoryName);
        if (intCtg >= 0) then
        begin
          strWork := Apps[intCtg].GetSortString + strWork;
        end;
      end;
    end;
  end;
  Result := strWork;
end;

procedure TAppItem.SaveToDB(DBID: Integer; DB: TIniFile; IconStream: tStream);
var
  Section: String;
  Istr:    TMemoryStream;
  Ico:     TIcon;
begin
  Section := format('APP-%3.3d', [DBID]);
  DB.WriteString(Section, 'Name', fName);
  if (fCaption <> fName) then
  begin
    DB.WriteString(Section, 'Caption', fCaption);
  end;
  DB.WriteString(Section, 'ExePath', fAppList.RelativePath(fExePath));
  if (not AnsiSameText(fHint, 'Location: ' + ExtractFileDir(fExePath))) then
  begin
    DB.WriteString(Section, 'Hint', fHint);
  end;
  if (fExecOnClose) then
  begin
    DB.WriteBool(Section, 'ExecOnClose', fExecOnClose);
  end;
  if (fAutoExec) then
  begin
    DB.WriteBool(Section, 'AutoExec', fAutoExec);
  end;
  if ( fExternalApp ) then
  begin
    DB.WriteBool(Section, 'ExternalApp', fExternalApp);
  end;
  if ( fExternalPath <> '' ) then
  begin
    DB.WriteString(Section, 'ExternalPath', fAppList.RelativePath(fExternalPath));
  end;
  if (not fVisible) then
  begin
    DB.WriteBool(Section, 'Visible', fVisible);
  end;
  if (fSortOrder <> 0) then
  begin
    DB.WriteInteger(Section, 'SortOrder', fSortOrder);
  end;
  if (fLevel <> 0) then
  begin
    DB.WriteInteger(Section, 'Level', fLevel);
  end;
  if (fCategoryName <> '') then
  begin
    DB.WriteString(Section, 'CategoryName', fCategoryName);
  end;
  if (fIsMenu) then
  begin
    DB.WriteBool(Section, 'IsMenu', fIsMenu);
  end;
  if (fMenuOpen) then
  begin
    DB.WriteBool(Section, 'MenuOpen', fMenuOpen);
  end;
  if (fExitExe <> '') then
  begin
    DB.WriteString(Section, 'ExitExe', fAppList.RelativePath(fExitExe));
  end;
  if (fExitParams <> '') then
  begin
    DB.WriteString(Section, 'ExitParams', fExitParams);
  end;
  if (fIconHandle <> 0) then
  begin
    if (fIconPath <> '') and ( ( not AnsiSameText(fIconPath, fExePath) ) or ( IsMenu) ) then
    begin
      DB.WriteString(Section, 'IconPath', fAppList.RelativePath(fIconPath));
    end;
    Ico        := TIcon.Create;
    Ico.Handle := IconHandle;
    Istr       := TMemoryStream.Create;
    Istr.Clear;
    Ico.SaveToStream(Istr);
    DB.WriteInteger(Section, 'IcoSize', Istr.Size);
    Istr.Seek(0, soFromBeginning);
    IconStream.CopyFrom(Istr, Istr.Size);
    Istr.Clear;
    Istr.Free;
    Ico.ReleaseHandle;
    Ico.Free;
  end;
  if (fOpenIconHandle <> 0) then
  begin
    if (fOpenIconPath <> '') then
    begin
      DB.WriteString(Section, 'OpenIconPath', fAppList.RelativePath(fOpenIconPath));
    end;
    Ico        := TIcon.Create;
    Ico.Handle := fOpenIconHandle;
    Istr       := TMemoryStream.Create;
    Istr.Clear;
    Ico.SaveToStream(Istr);
    DB.WriteInteger(Section, 'OpenIcoSize', Istr.Size);
    Istr.Seek(0, soFromBeginning);
    IconStream.CopyFrom(Istr, Istr.Size);
    Istr.Clear;
    Istr.Free;
    Ico.ReleaseHandle;
    Ico.Free;
  end;
end;

function TAppItem.GetPAFFile: String;
begin
  if fExternalApp then
    Result := fExternalPath
  else
    Result := ChangeFileExt(fExePath, '.pafdata');
end;

function TAppItem.PAFWriteable: Boolean;
begin
  Result := True;
  if FileExists(GetPAFFile) then
  begin
    if FileIsReadOnly(GetPAFFile) then
    begin
      Result := False;
    end;
  end;
end;

function TAppItem.ItemIndex: Integer;
var
  I: Integer;
begin
  Result := -1;
  if Assigned(fAppList) then
  begin
    for I := 0 to fAppList.Count - 1 do
    begin
      if fAppList.fAppStringList.Objects[I] = Self then
      begin
        Result := I;
        Break;
      end;
    end;
  end;
end;

function TAppItem.SortKey: String;
  // String to Use for INI Files SortOrder Key
begin
  if fIsMenu then
  begin
    Result := 'Category-' + StringReplace(fName, '=', '|eq|', [rfReplaceAll, rfIgnoreCase]);
  end
  else if fExternalApp then
  begin
    Result := 'External-' + ExtractFileName(GetPAFFile);
  end
  else
  begin
    Result := StringReplace(fAppList.RelativePath(fExePath), '=', '|eq|', [rfReplaceAll, rfIgnoreCase]);
  end;
end;

constructor TAppItemList.Create;
begin
  inherited Create;
  fDefaultIconPath   := Application.ExeName;
  fDefaultOpenIconPath := '';
  fBasePath          := ExtractFilePath(ExpandUNCFileName(Application.ExeName));
  fDataFile          := ChangeFileExt(Application.ExeName, '.ini');
  fAppDBname         := ChangeFileExt(Application.ExeName, '.fsa');
  fExtAppDir         := ExtractFilePath(Application.ExeName) + 'Data\OtherApp';
  fShowAll           := False;
  fModified          := False;
  fAppStringList     := TStringList.Create;
  fAppStringList.Sorted := True;
  fAppStringList.Duplicates := dupAccept;
  fReadOnly          := False;
  fMenuIndent        := 10;
  fUpdateCount       := 0;
  fAppFont           := TFont.Create;
  fAppFont.Color     := clBlack;
  fAppFont.Size      := 8;
  fAppFont.Name      := 'Tahoma';
  fAppFont.Style     := [fsBold];
  fMenuFont          := TFont.Create;
  fMenuFont.Color    := clBlack;
  fMenuFont.Size     := 8;
  fMenuFont.Name     := 'Tahoma';
  fMenuFont.Style    := [fsBold];
  fMenuOpenFont      := TFont.Create;
  fMenuOpenFont.Color := clBlack;
  fMenuOpenFont.Size := 8;
  fMenuOpenFont.Name := 'Tahoma';
  fMenuOpenFont.Style := [fsBold];
end;

destructor TAppItemList.Destroy;
begin
  Clear;
  fAppStringList.Free;
  fMenuOpenFont.Free;
  fMenuFont.Free;
  fAppFont.Free;
  //  fCategories.Free;
  inherited Destroy;
end;

procedure TAppItemList.Clear;
begin
  while fAppStringList.Count > 0 do
  begin
    Apps[0].Free;
//    fAppStringList.Objects[0].Free;
    fAppStringList.Delete(0);
  end;
  fAppStringList.Clear;
  //    fCategories.Clear;
end;

procedure TAppItemList.Delete(ItemIndex: Integer);
begin
  if (ItemIndex >= 0) and (ItemIndex < Count) then
  begin
    fAppStringList.Objects[ItemIndex].Free;
    fAppStringList.Delete(ItemIndex);
  end;
end;

procedure TAppItemList.SetModified(bolValue: Boolean);
begin
  if bolValue <> fModified then
  begin
    FixLevels;
    fModified := bolValue;
    if fModified then
    begin
      if fUpdateCount = 0 then
      begin
        if Assigned(fOnChanged) then
        begin
          fOnChanged(Self);
        end;
      end;
    end;
  end;
end;

procedure TAppItemList.SetDefaultIcon(strValue: String);
begin
  if not AnsiSameText(strValue, fDefaultIconPath) then
  begin
    BeginUpdate;
    if fDefaultIconHandle <> 0 then
    begin
      DestroyIcon(fDefaultIconHandle);
      fDefaultIconHandle := 0;
    end;
    fDefaultIconHandle := GetIconFromFile(strValue);
    if fDefaultIconHandle <> 0 then
    begin
      fDefaultIconPath := strValue;
    end else begin
      fDefaultIconHandle := GetIconFromFile(Application.ExeName);
      strValue := Application.ExeName;
    end;
    EndUpdate;
  end;
end;

procedure TAppItemList.SetDefaultOpenIcon(strValue: String);
begin
  if not AnsiSameText(strValue, fDefaultOpenIconPath) then
  begin
    BeginUpdate;
    if fDefaultOpenIconHandle <> 0 then
    begin
      DestroyIcon(fDefaultOpenIconHandle);
      fDefaultOpenIconHandle := 0;
    end;
    fDefaultOpenIconPath := '';
    fDefaultOpenIconHandle := GetIconFromFile(strValue);
    if fDefaultOpenIconHandle <> 0 then
    begin
      fDefaultOpenIconPath := strValue;
    end;
    EndUpdate;
  end;
end;

procedure TAppItemList.SetUpdateCount(intValue: Integer);
begin
  if fUpdateCount <> intValue then
  begin
    if intValue > 0 then
    begin
      fUpdateCount := intValue;
    end
    else
    begin
      fUpdateCount := 0;
    end;
  end;
  if fUpdateCount = 0 then
  begin
    if Assigned(fOnChanged) then
    begin
      fOnChanged(Self);
    end;
  end;
end;

function TAppItemList.Count: Integer;
begin
  Result := fAppStringList.Count;
end;

function TAppItemList.IsVisible(ItemIndex: Integer): Boolean;
var
  I: Integer;
  L: Integer;
begin
  Result := False;
  if (ItemIndex < 0) or (ItemIndex >= fAppStringList.Count) then
  begin
    Exit;
  end;
  if not Apps[ItemIndex].Visible and not ShowAll then
  begin
    Exit;
  end;
  if Apps[ItemIndex].Level = 0 then
  begin
    Result := True;
    Exit;
  end;
  L := Apps[ItemIndex].Level;
  I := ItemIndex;
  while L > 0 do
  begin
    while Apps[I].Level >= L do
    begin
      Dec(I);
    end;
    if (not Apps[I].Visible and not ShowAll) or (not Apps[I].MenuOpen) then
    begin
      Exit;
    end; // Menu is closed or hidden then app in menu is not visible
    L := Apps[I].Level; // Set L to check higher level
  end;
  Result := True;
end;

function TAppItemList.FirstVisible: Integer;
var
  I: Integer;
begin
  Result := -1;
  if Count > 0 then
  begin
    for I := 0 to fAppStringList.Count - 1 do
    begin
      if IsVisible(I) then
      begin
        Result := I;
        Break;
      end;
    end;
  end;
end;

function TAppItemList.PrevVisible(From: Integer): Integer;
var
  I: Integer;
begin
  Result := -1;
  if From >= Count then
  begin
    From := Count - 1;
  end; // Max val coming in has to be count-1
  if From > 0 then
  begin
    for I := From - 1 downto 0 do
    begin
      if IsVisible(I) then
      begin
        Result := I;
        Break;
      end;
    end;
  end;
end;

function TAppItemList.NextVisible(From: Integer): Integer;
var
  I: Integer;
begin
  Result := -1;
  if From < -1 then
  begin
    From := 0;
  end;
  if From < Count - 1 then
  begin
    for I := From + 1 to Count - 1 do
    begin
      if IsVisible(I) then
      begin
        Result := I;
        Break;
      end;
    end;
  end;
end;

function TAppItemList.LastVisible: Integer;
var
  I: Integer;
begin
  Result := -1;
  if Count > 0 then
  begin
    for I := fAppStringList.Count - 1 downto 0 do
    begin
      if IsVisible(I) then
      begin
        Result := I;
        Break;
      end;
    end;
  end;
end;

function TAppItemList.AtTop: Boolean;
begin
  Result := (FirstVisible >= ButtonsArray[0].Tag);
end;

function TAppItemList.AtBottom: Boolean;
var
  I: Integer;
  C: Integer;
begin
  I := LastVisible;
  C := 1;
  while (C < Length(ButtonsArray)) and (I > ButtonsArray[0].Tag) do
  begin
    I := PrevVisible(I);
    Inc(C);
  end;
  Result := (C < Length(ButtonsArray)) or (I <= ButtonsArray[0].Tag);
end;

function TAppItemList.GetAppItem(Index: Integer): TAppItem;
begin
  Result := TAppItem(fAppStringList.Objects[Index]);
end;

function TAppItemList.GetCtgByName(CtgName: String): Integer;
var
  I: Integer;
begin
  Result := -1;
  if CtgName = '' then
  begin
    Exit;
  end;
  if Count > 0 then
  begin
    for I := 0 to Count - 1 do
    begin
      if Apps[I].IsMenu then
      begin
        if AnsiSameText(Apps[I].fName, CtgName) then
        begin
          Result := I;
          Break;
        end;
      end;
    end;
  end;
end;

procedure TAppItemList.ToggleMenu(ItemIndex: Integer);
begin
  if (ItemIndex >= 0) and (ItemIndex < Count) then
    Apps[ItemIndex].MenuOpen := not Apps[ItemIndex].MenuOpen;
end;

function TAppItemList.AddApp(ExePath: String): Integer;
var
  App:    TAppItem;
  intItem: Integer;
begin
  Result := -1;
  App := TAppItem.CreateFromExe(ExePath, Self, intItem);
  if AnsiSameText(App.Name, 'OpenOffice.org Base Portable') or ( App.ExePath = '' ) then
  begin
    Apps[intItem].Free;  // Has to be this instead of app.free or it doesn't get freed
    fAppStringList.Delete(intItem);
    // Kludge to force ignoring Openoffice Base Program
  end else begin
    Result := intItem;
    fModified := True;
  end;
end;

function TAppItemList.AddApp(DBID: Integer; DB: TIniFile; IconStream: TStream): Integer;
var
  App:    TAppItem;
  intItem: Integer;
begin
  Result := -1;
  App := TAppItem.CreateFromDB(DBID, DB, IconStream, Self, intItem);
  if Assigned(App) then
    Result := intItem;
end;

function TAppItemList.AddCategory(strName: String; strCaption: String;
  strCategory: String; bolIsOpen: Boolean; intSortOrder: Integer; strIcoPath: String;
  strOpenIcoPath: String): Integer;
var
  App:    TAppItem;
  intCtg: Integer;
  intItem: Integer;
begin
  Result := -1;
  intCtg := GetCtgByName(strName);
  if intCtg = -1 then
  begin
    BeginUpdate;
    App := TAppItem.CreateNewCategory(strName, strCaption, strCategory, strIcoPath, strOpenIcoPath, Self, intItem);
    App.MenuOpen := BolIsOpen;
    fModified := True;
    FixLevels;
    Result := App.ItemIndex;
    EndUpdate;
  end;
end;

procedure TAppItemList.DelCategory(strName: String);
begin
  DelCategory(GetCtgByName(strName));
end;

procedure TAppItemList.DelCategory(ItemIndex: Integer);
var
  I: Integer;
begin
  if (ItemIndex >= 0) and (ItemIndex < Count) then
  begin
    if Apps[ItemIndex].IsMenu then
    begin
      BeginUpdate;
      I := ItemIndex + 1;
      while (I < Count) do
      begin
        if (Apps[I].Level > Apps[ItemIndex].Level) then
        begin
          Apps[I].fLevel := Apps[I].Level - 1;
          Inc(I);
        end else
        begin
          I := Count;
        end;
      end;
      fAppStringList.Objects[ItemIndex].Free;
      fAppStringList.Delete(ItemIndex);
      FixLevels;
      EndUpdate;
    end;
  end;
end;

procedure TAppItemList.SaveToDB;
begin
  SaveToDB(fAppDBname);
end;

procedure TAppItemList.SaveToDB(DBFileName: String);
var
  DB: TIniFile;
  IconFileName: String;
  IconStream: TFileStream;
  I: Integer;
begin
  if FileExists(DBFileName) then
  begin
    DeleteFile(DBFileName);
  end;
  IconFileName := ChangeFileExt(DBFileName, '.fsi');
  if FileExists(IconFileName) then
  begin
    DeleteFile(IconFileName);
  end;
  if (Count > 0) then
  begin
    DB := TIniFile.Create(DBFileName);
    DB.WriteString('DB', 'Version', AppDBVersion);
    DB.WriteInteger('DB', 'Count', Count);
    IconStream := TFileStream.Create(IconFileName, fmCreate);
    for I := 0 to Count - 1 do
    begin
      Apps[I].SaveToDB(I, DB, IconStream);
    end;
    DB.WriteInteger('DB', 'TotalIconSize', IconStream.Size);
    IconStream.Free;
    DB.Free;
  end;
end;

function TAppItemList.LoadFromDB: Boolean;
begin
  Result := LoadFromDB(fAppDBname);
end;

function TAppItemList.LoadFromDB(DBFileName: String): Boolean;
var
  DB: TIniFile;
  IconFileName: String;
  IconStream: TFileStream;
  I: Integer;
  C: Integer;
  S: String;
begin
  Result := False;
  Clear;
  if FileExists(DBFileName) then
  begin
    IconFileName := ChangeFileExt(DBFileName, '.fsi');
    if FileExists(IconFileName) then
    begin
      DB := TIniFile.Create(DBFileName);
      try // Used so an Exit will close the file properly
        S := DB.ReadString('DB', 'Version', '');
          // Minor Version Changes are OK
        if (Copy(S, 0, Pos('.', S)) <> Copy(AppDBVersion, 0, Pos('.', AppDBVersion))) then
        begin
          Exit;
        end;
        IconStream := TFileStream.Create(IconFileName, fmOpenRead);
        try
          I := IconStream.Size;
          // Fail if IconFile Size is wrong
          if (DB.ReadInteger('DB', 'TotalIconSize', 0) <> I) then
          begin
            Exit;
          end;
          C := DB.ReadInteger('DB', 'Count', 0);
          if (C > 0) then
          begin
            for I := 0 to C - 1 do
            begin
              AddApp(I, DB, IconStream);
            end;
          end;
          Result := True;
        finally
          IconStream.Free;
        end;
      finally
        DB.Free;
      end;
    end;
  end;
  fModified := False;
end;

function TAppItemList.LoadFromPath(strPortableAppsPath: String; Excludes: TStrings): Boolean;
var
  tsrchDirectory:    TSearchRec;
  tsrchSubDirectory: TSearchRec;
  strTempSubDir:     String;
begin
  Result := False;
  // Always ignore current and parent directories
  Excludes.Add('.');
  Excludes.Add('..');
  if FindFirst(IncludeTrailingPathDelimiter(strPortableAppsPath) + '*',
    faDirectory, tsrchDirectory) = 0 then
  begin
    repeat
      strTempSubDir := tsrchDirectory.Name;
      // Filter Out Excluded Directories
      if (Excludes.IndexOf(strTempSubDir) = -1) then
      begin
        strTempSubDir := IncludeTrailingPathDelimiter(
          IncludeTrailingPathDelimiter(strPortableAppsPath) + strTempSubDir);
        if FindFirst(strTempSubDir + '*.exe', faAnyFile, tsrchSubDirectory) = 0 then
        begin
          repeat
            // Ensure it's not a directory with a .exe name
            if ((tsrchSubDirectory.Attr and faDirectory) = 0) then
            begin
              AddApp(strTempSubDir + tsrchSubDirectory.Name);
              Result := True;
            end;
          until FindNext(tsrchSubDirectory) <> 0;
          FindClose(tsrchSubDirectory);
        end;
      end;
    until FindNext(tsrchDirectory) <> 0;
    FindClose(tsrchDirectory);
  end;
  if DirectoryExists(fExtAppDir) then
  begin
    strTempSubDir := IncludeTrailingPathDelimiter(fExtAppDir);
    if FindFirst(strTempSubDir + '*.pafdata', faAnyFile, tsrchSubDirectory) = 0 then
    begin
      repeat
        // Ensure it's not a directory with a .pafdata name
        if ((tsrchSubDirectory.Attr and faDirectory) = 0) then
        begin
          AddApp(strTempSubDir + tsrchSubDirectory.Name);
              Result := True;
        end;
      until FindNext(tsrchSubDirectory) <> 0;
      FindClose(tsrchSubDirectory);
    end;
  end;
  fModified := True;
end;

function TAppItemList.LoadCategories: Integer;
var
  I:           Integer;
  intNew:      Integer;
  strSection:  String;
  strIcon:     String;
  strOpenIcon: String;
  strName:     String;
  strCaption:  String;
  strCategory: String;
begin
  Result   := 0;
  with TIniFile.Create(DataFile) do
  begin
    I          := 0;
    strSection := Format('Ctg_%d', [I]);
    while SectionExists(strSection) do
    begin
      strName     := ReadString(strSection, 'Name', strSection);
      strCaption  := ReadString(strSection, 'Caption', strName);
      strCategory := ReadString(strSection, 'Category', '');
      // Custom Icons
      strIcon     := FullPath(ReadString(strSection, 'Icon', ''));
      strOpenIcon := FullPath(ReadString(strSection, 'OpenIcon', ''));
      intNew := AddCategory(strName, strCaption, strCategory, False, 0, strIcon, strOpenIcon);
      if (intNew <> -1) then
      begin
        Apps[intNew].fHint := ReadString(strSection, 'Hint', '');
        Apps[intNew].fSortOrder := ReadInteger('SortOrder', Apps[intNew].SortKey, 0);
      end;
      Inc(I);
      strSection := Format('Ctg_%d', [I]);
    end;
    Free;
  end;
end;

procedure TAppItemList.SaveCategories;
var
  I:          Integer;
  strSection: String;
  C:          Integer;
begin
  C := 0;
  with TIniFile.Create(DataFile) do
  begin
    for I := 0 to Count - 1 do
    begin
      if Apps[I].IsMenu then
      begin
        strSection := Format('Ctg_%d', [C]);
        EraseSection(strSection);
        WriteString(strSection, 'Name', Apps[I].Name);
        WriteString(strSection, 'Caption', Apps[I].Caption);
        if (Apps[I].CategoryName <> '') then
        begin
          WriteString(strSection, 'Category', Apps[I].CategoryName);
        end;
        if (Trim(Apps[I].Hint) <> '' ) then
          WriteString(strSection, 'Hint', Apps[I].Hint);
        if (Apps[I].IconPath <> '' ) then
          WriteString(strSection, 'Icon', RelativePath(Apps[I].IconPath));
        if (Apps[I].OpenIconPath <> '') then
          WriteString(strSection, 'OpenIcon', RelativePath(Apps[I].OpenIconPath));
        Inc(C);
      end;
    end;
    while SectionExists(Format('Ctg_%d', [C])) do
    begin
      EraseSection(Format('Ctg_%d', [C]));
      Inc(C);
    end;
    Free;
  end;
end;

procedure TAppItemList.Apply(intIndex: Integer; intTarget: Integer);
var
  brshIcon: hBrush;
  hndIcon:  HIcon;
begin
  if (IsVisible(intIndex)) then
  begin
    hndIcon := Apps[intIndex].IconHandle;
    if Apps[intIndex].IsMenu then
    begin
      if Apps[intIndex].MenuOpen then
      begin
        ButtonsArray[intTarget].Font := MenuOpenFont;
        if ( Apps[intIndex].OpenIconHandle <> 0 ) then
        begin
          hndIcon := Apps[intIndex].OpenIconHandle;
        end else if ( Apps[intIndex].IconHandle <> 0 ) then
          begin
            hndIcon := Apps[intIndex].IconHandle;
          end else begin
            hndIcon := fDefaultOpenIconHandle;
          end;
      end else
      begin
        ButtonsArray[intTarget].Font := MenuFont;
      end;
    end else
    begin
      ButtonsArray[intTarget].Font := AppFont;
    end;
    if hndIcon = 0 then
      hndIcon := fDefaultIconHandle;
    with ButtonsArray[intTarget] do
    begin
      Glyph.TransparentColor := TransColor;
      Margin := Apps[intIndex].Level * MenuIndent;
      Caption := Apps[intIndex].Caption;
      Hint := Apps[intIndex].Hint;
      brshIcon := CreateSolidBrush(clWhite);
      Glyph.Canvas.Rectangle(0, 0, 16, 16);
      DrawIconEx(Glyph.Canvas.Handle, 0, 0, hndIcon, 16, 16, 0, brshIcon, DI_NORMAL);
      Glyph.Canvas.Refresh;
      DeleteObject(brshIcon);
      Visible := True;
      Tag := intIndex;
      BringToFront;
    end;
  end else
  begin
    ButtonsArray[intTarget].Visible := False;
    ButtonsArray[intTarget].Tag := intIndex;
  end;
end;


procedure TAppItemList.DrawButtons;
var
  I:       Integer;
  intItem: Integer;
begin
  if Length(ButtonsArray) > 0 then
  begin
    intItem := ButtonsArray[0].Tag;
    if not IsVisible(intItem) then
    begin
      intItem := NextVisible(intItem);
    end;
    for I := 0 to Length(ButtonsArray) - 1 do
    begin
      Apply(intItem, I);
      if IsVisible(intItem) then
      begin
        intItem := NextVisible(intItem);
      end;
    end;
  end;
end;

procedure TAppItemList.ForceVisible(ItemIndex: Integer);
var
  I: Integer;
  C: Integer;
begin
  // First make sure item and parents are visible
  if not Apps[ItemIndex].Visible then ShowAll := True;
  I := ItemIndex;
  while ( I >= 0 ) and ( Apps[I].Level <> 0 ) do
  begin
    I := GetCtgByName(Apps[I].CategoryName);
    If not Apps[I].Visible then ShowAll := True;
    Apps[I].MenuOpen := True;
  end;
  if ItemIndex < ButtonsArray[0].Tag then
  begin
    BeginUpdate;
    ButtonsArray[0].Tag := ItemIndex;
    EndUpdate;
    Exit;
  end;
  I := ButtonsArray[0].Tag;
  for C := 1 to Length(ButtonsArray) - 1 do
  begin
    if I = ItemIndex then
      Break;
    I := NextVisible(I);
  end;
  BeginUpdate;
  while ( I <> ItemIndex ) and ( I >= 0 )do
  begin
    I := NextVisible(I);
    Scroll(1);
  end;
  EndUpdate;
end;

procedure TAppItemList.ShowSubItems(ItemIndex: Integer);
var
  I: Integer;
  P: Integer;
  C: Integer;
begin
  if not Apps[ItemIndex].IsMenu then
  begin
    Exit;
  end;
  if ButtonsArray[Length(ButtonsArray) - 1].Visible then
  begin
    // Last item is visible so we may have to scroll down
    if ButtonsArray[0].Tag = ItemIndex then
    begin
      Exit;
    end;
    if not Apps[ItemIndex].IsMenu then
    begin
      Exit;
    end;
    if not Apps[ItemIndex].MenuOpen then
    begin
      Exit;
    end;
    C := 0;
    I := NextVisible(ItemIndex);
    while (I > 0) and (Apps[I].Level > Apps[ItemIndex].Level) do
    begin
      Inc(C);
      I := NextVisible(I);
    end;
    if (Length(ButtonsArray) <= C + 1) then
    begin // Move Menu to top
      BeginUpdate;
      ButtonsArray[0].Tag := ItemIndex;
      EndUpdate;
    end;
    P := -1;
    for I := 0 to Length(ButtonsArray) - 1 do
    begin
      if ButtonsArray[I].Tag = ItemIndex then
      begin
        P := I;
        Break;
      end;
    end;
    if P = -1 then
    begin
      Exit;
    end; // Menu top not visible
    if (Length(ButtonsArray) - P > C) then
    begin
      Exit;
    end; // All Visible
    BeginUpdate;
    I := P + 1 - Length(ButtonsArray) + C;
    if I > P then
    begin
      I := P;
    end;
    Scroll(I);
    EndUpdate;
  end else
  begin
    // Last item isn't visible so we may have to scroll up
    if Apps[ItemIndex].MenuOpen then
    begin
      Exit;
    end;
    if FirstVisible = ButtonsArray[0].Tag then
    begin
      Exit;
    end;
    if not ButtonsArray[0].Visible then
    begin
      Exit;
    end;
    C := 0;
    I := Length(ButtonsArray) - 1;
    while not ButtonsArray[I + C].Visible do
    begin
      Dec(C);
    end;
    BeginUpdate;
    Scroll(C);
    EndUpdate;
  end;
end;

procedure TAppItemList.FixLevels;
var
  I:      Integer;
  intCtg: Integer;
begin
  if (Count > 0) then
  begin
    BeginUpdate;
    fAppStringList.Sorted := False;
    for I := 0 to Count - 1 do
    begin
      fAppStringList[I] := Apps[I].GetSortString;
      if Apps[I].CategoryName = '' then
      begin
        Apps[I].fLevel := 0
      end else begin
        intCtg      := GetCtgByName(Apps[I].CategoryName);
        if intCtg >= 0 then
        begin
          Apps[I].fLevel := Apps[intCtg].Level + 1;
        end else
        begin
          Apps[I].fLevel := 0;
        end;
      end;
    end;
    fAppStringList.Sort;
    fAppStringList.Sorted := True;
    EndUpdate;
  end;
end;

procedure TAppItemList.SetSortString(intItemIndex: Integer; strValue: String);
begin
  fAppStringList.Sorted := False;
  fAppStringList[intItemIndex] := strValue;
  fAppStringList.Sort;
  fAppStringList.Sorted := True;
end;

procedure TAppItemList.ReSort;
begin
  if fLastSort <> astManual then
    ReSort(fLastSort);
end;

procedure TAppItemList.ReSort(astSortType: TAppSortType);
var
  I: Integer;
  S: Integer;
begin
  BeginUpdate;
  with TIniFile.Create(fDataFile) do
  begin
    EraseSection('SortOrder');
    Free;
  end;
  if (Count > 0) then
  for I := 0 to Count - 1 do
  begin
    S := 0;
    if Apps[I].isMenu then
    begin
      if astSortType = astAlphaCtgLast then
      begin
        S := 1;
      end;
    end else begin
      if astSortType = astAlphaCtgFirst then
      begin // Categories First
        S := 1;
      end;
    end;
    Apps[I].SortOrder := S;
  end;
  FixLevels;
  EndUpdate;
  fLastSort := astSortType;
end;


function TAppItemList.PrepareSort(ItemIndex: Integer; slWork: TStringList): Integer;
  // Put All Items at this level in the menu into a stringlist in preparation for sort changes
var
  I: Integer;
  A: Integer;
begin
  Result := -1;
  slWork.Clear;
  I      := ItemIndex;
  while (I > 0) and (Apps[I - 1].Level >= Apps[ItemIndex].Level) do
  begin
    Dec(I);
  end; // Get First Item in Current Level
  while (I < Count) do
  begin
    if (Apps[I].Level = Apps[ItemIndex].Level) then
    begin
      A := slWork.AddObject(Apps[I].Caption, Apps[I]);
      if (I = ItemIndex) then
      begin
        Result := A;
      end;
    end else if (Apps[I].Level < Apps[ItemIndex].Level) then
    begin
      I := Count;
    end;
    Inc(I);
  end;
end;

function TAppItemList.CanSortUp(ItemIndex: Integer): Boolean;
var
  I: Integer;
begin
  Result := False;
  if (ItemIndex > 0) and (ItemIndex < Count) then
  begin
    I := ItemIndex - 1;
    while (I >= 0) and (not Result) and (Apps[I].Level >= Apps[ItemIndex].Level) do
    begin
      if Apps[I].Level = Apps[ItemIndex].Level then
      begin
        Result := True;
      end
      else
      begin
        Dec(I);
      end;
    end;
  end;
end;

function TAppItemList.CanSortDown(ItemIndex: Integer): Boolean;
var
  I: Integer;
begin
  Result := False;
  if (ItemIndex >= 0) and (ItemIndex < Count - 1) then
  begin
    I := ItemIndex + 1;
    while (I < Count) and (not Result) and (Apps[I].Level >= Apps[ItemIndex].Level) do
    begin
      if Apps[I].Level = Apps[ItemIndex].Level then
      begin
        Result := True;
      end
      else
      begin
        Inc(I);
      end;
    end;
  end;
end;

function TAppItemList.SortUp(ItemIndex: Integer): Integer;
var
  I:   Integer;
  L:   TStringList;
  App: TAppItem;
  OrigApp: TAppItem;
begin
  Result := ItemIndex;
  if (ItemIndex > 0) and (ItemIndex < Count) then
  begin
    OrigApp := Apps[ItemIndex]; // Save original app
    L := TStringList.Create;
    I := PrepareSort(ItemIndex, L);
    if (I > 0) and (I < L.Count) then
    begin
      BeginUpdate;
      App          := TAppItem(L.Objects[I]);
      L.Objects[I] := L.Objects[I - 1];
      L.Objects[I - 1] := App;
      FinalizeSort(L);
      EndUpdate;
    end;
    L.Free;
    Result := OrigApp.ItemIndex; // Return New App Position
    ForceVisible(Result);
  end;
end;

function TAppItemList.SortDown(ItemIndex: Integer): Integer;
var
  I:   Integer;
  L:   TStringList;
  App: TAppItem;
  OrigApp: TAppItem;
begin
  Result := ItemIndex;
  if (ItemIndex >= 0) and (ItemIndex < Count - 1) then
  begin
    OrigApp := Apps[ItemIndex]; // Save original app
    L := TStringList.Create;
    I := PrepareSort(ItemIndex, L);
    if (I >= 0) and (I < L.Count - 1) then
    begin
      BeginUpdate;
      App := TAppItem(L.Objects[I]);
      L.Objects[I] := L.Objects[I + 1];
      L.Objects[I + 1] := App;
      FinalizeSort(L);
      EndUpdate;
    end;
    L.Free;
    Result := OrigApp.ItemIndex; // Return New App Position
    ForceVisible(Result);
  end;
end;

function TAppItemList.SortTop(ItemIndex: Integer): Integer;
var
  I:   Integer;
  L:   TStringList;
  App: TAppItem;
  OrigApp: TAppItem;
begin
  Result := ItemIndex;
  if (ItemIndex > 0) and (ItemIndex < Count) then
  begin
    OrigApp := Apps[ItemIndex]; // Save original app
    L := TStringList.Create;
    I := PrepareSort(ItemIndex, L);
    if (I > 0) and (I < L.Count) then
    begin
      BeginUpdate;
      App := TAppItem(L.Objects[I]);
      while I > 0 do
      begin
        L.Objects[I] := L.Objects[I - 1];
        Dec(I);
      end;
      L.Objects[I] := App;
      FinalizeSort(L);
      EndUpdate;
    end;
    L.Free;
    Result := OrigApp.ItemIndex; // Return New App Position
    ForceVisible(Result);
  end;
end;

function TAppItemList.SortBottom(ItemIndex: Integer): Integer;
var
  I:   Integer;
  L:   TStringList;
  App: TAppItem;
  OrigApp: TAppItem;
begin
  Result := ItemIndex;
  if (ItemIndex >= 0) and (ItemIndex < Count - 1) then
  begin
    OrigApp := Apps[ItemIndex]; // Save original app
    L := TStringList.Create;
    I := PrepareSort(ItemIndex, L);
    if (I >= 0) and (I < L.Count - 1) then
    begin
      BeginUpdate;
      App := TAppItem(L.Objects[I]);
      while I < L.Count - 1 do
      begin
        L.Objects[I] := L.Objects[I + 1];
        Inc(I);
      end;
      L.Objects[I] := App;
      FinalizeSort(L);
      EndUpdate;
    end;
    L.Free;
    Result := OrigApp.ItemIndex; // Return New App Position
    ForceVisible(Result);
  end;
end;

procedure TAppItemList.FinalizeSort(slWork: TStringList);
// Set the sortorder for items in sorting list, save to ini
var
  I: Integer;
  O: Integer;
begin
  with TIniFile.Create(DataFile) do
  begin
    O := 0;
    DeleteKey('SortOrder', TAppItem(slWork.Objects[0]).SortKey);
    TAppItem(slWork.Objects[0]).SortOrder := 0;
    for I := 1 to slWork.Count - 1 do
    begin
      if AnsiCompareText(TAppItem(slWork.Objects[I - 1]).Caption,
        TAppItem(slWork.Objects[I]).Caption) >= 0 then
      begin
        Inc(O);
      end;
      TAppItem(slWork.Objects[I]).SortOrder := O;
      DeleteKey('SortOrder', TAppItem(slWork.Objects[I]).SortKey);
      if O > 0 then
        WriteInteger('SortOrder', TAppItem(slWork.Objects[I]).SortKey, O);
    end;
    Free;
  end;
  FixLevels;  // reset sort strings
end;

procedure TAppItemList.Scroll(intSteps: Integer);
var
  I:          Integer;
  intTopItem: Integer;
begin
  if Length(ButtonsArray) > 0 then
  begin
    intTopItem := ButtonsArray[0].Tag;
    while intSteps <> 0 do
    begin
      if intSteps < 0 then
      begin
        if not AtTop then
        begin
          I := PrevVisible(ButtonsArray[0].Tag);
          if I >= 0 then ButtonsArray[0].Tag := I;
          Inc(intSteps);
        end else
        begin
          intSteps := 0;
        end;
      end else
      begin
        if not AtBottom then
        begin
          I := NextVisible(ButtonsArray[0].Tag);
          if I >= 0 then ButtonsArray[0].Tag := I;
          Dec(intSteps);
        end else
        begin
          intSteps := 0;
        end;
      end;
    end;
    if ButtonsArray[0].Tag <> intTopItem then
    begin
      BeginUpdate;
      EndUpdate; // put in to automate redrawing
    end;
  end;
end;

procedure TAppItemList.CategoryMenu(ItemIndex: Integer; mnuParent: TMenuItem);
var
  M: TMenuItem;
  I: Integer;
  P: Integer;
begin
  while (mnuParent.Count > 0) and (mnuParent.Items[0].Caption <> '-') do
  begin
    mnuParent.Delete(0);
  end;
  P := 0;
  mnuParent.Tag := ItemIndex;
  for I := 0 to Count - 1 do
  begin
    if (Apps[I].IsMenu) then
    begin
      M         := TMenuItem.Create(mnuParent);
      M.Caption := StringOfChar(' ', Apps[I].Level * 2) + Apps[I].Caption;
      M.Tag     := I;
      M.OnClick := mnuCategoryClick;
      M.Checked := AnsiSameText(Apps[I].Name, Apps[ItemIndex].CategoryName);
      mnuParent.Insert(P, M);
      Inc(P);
    end;
  end;
end;

procedure TAppItemList.mnuCategoryClick(Sender: TObject);
// Procedure linked to category menu items
var
  ItemIndex: Integer;
  CtgIndex:  Integer;
  I:         Integer;
  App:       TAppItem;
begin
  CtgIndex := TControl(Sender).Tag;
  if CtgIndex >= 0 then
  begin
    ItemIndex := TMenuItem(Sender).Parent.Tag;
    App := Apps[ItemIndex];
    if (ItemIndex >= 0) and (ItemIndex < Count) and (ItemIndex <> CtgIndex) then
    begin
      if TMenuItem(Sender).Checked then
      begin
        BeginUpdate;
        Apps[ItemIndex].CategoryName := '';
        ForceVisible(App.ItemIndex);
        EndUpdate;
      end else
      begin
        I := CtgIndex;
        while (I <> -1) and (I <> ItemIndex) do
        begin
          I := GetCtgByName(Apps[I].CategoryName);
        end;
        if (I <> ItemIndex) then  // to prevent linking under own tree
        begin
          BeginUpdate;
          Apps[ItemIndex].CategoryName := Apps[CtgIndex].Name;
          ForceVisible(App.ItemIndex);
          EndUpdate;
        end;
      end;
    end;
  end;
end;

procedure TAppItemList.BeginUpdate;
begin
  UpdateCount := fUpdateCount + 1;
end;

procedure TAppItemList.EndUpdate;
begin
  UpdateCount := fUpdateCount - 1;
end;

procedure TAppItemList.AutoExec;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    if not Apps[I].IsMenu then
    begin
      if Apps[I].AutoExec then
      begin
        Apps[I].AppExecute;
      end;
    end;
  end;
end;

function TAppItemList.AppExecute(ItemIndex: Integer): Boolean;
begin
  Result := False;
  if (ItemIndex >= 0) and (ItemIndex < Count) then
  begin
    Result := Apps[ItemIndex].AppExecute;
  end;
end;

procedure TAppItemList.SetFontSize(intSize: Integer);
begin
  AppFont.Size      := intSize;
  MenuFont.Size     := intSize;
  MenuOpenFont.Size := intSize;
end;

procedure TAppItemList.ExitExec;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    if not Apps[I].IsMenu then
    begin
      if Apps[I].ExecOnClose then
      begin
        Apps[I].ExitExecute;
      end;
    end;
  end;
end;

procedure TAppItemList.ToggleAll(bolOpen: Boolean);
var
  I: Integer;
begin
  BeginUpdate;
  if (Count > 0) then
  for I := 0 to Count - 1 do
  begin
    if Apps[I].IsMenu then
      Apps[I].MenuOpen := bolOpen;
  end;
  ButtonsArray[0].Tag := FirstVisible;
  EndUpdate;
end;

function TAppItemList.AddExternalApp(strExePath: String): Integer;
var
  strExe: String;
  strPaf: String;
  intUniq: Integer;
  App: TAppItem;
begin
  Result := -1;
  strExe := RelativePath(strExePath);
  if AnsiSameText(strExe, strExePath) then
    Exit; // Must be on same drive
  if not DirectoryExists(fExtAppDir) then
  begin
    ForceDirectories(fExtAppDir);
  end;
  strPaf := fExtAppDir + '\' + ChangeFileExt(ExtractFileName(strExe), '.pafData');
  intUniq := 0;
  while FileExists(strPaf) do
  begin
    Inc(intUniq);
    strPaf := ExtractFilePath(strPaf) + ChangeFileExt(ExtractFileName(strExe), '_' + IntToStr(intUniq) + '.pafData');
  end;
  with TIniFile.Create(strPaf) do
  begin
    WriteString('PAFDATA', 'ExePath', strExe);
    Free;
  end;
  BeginUpdate;
  App := Apps[AddApp(strPaf)];
  ReSort;
  Result := App.ItemIndex;
  EndUpdate;
end;

function TAppItemList.FullPath(strPath: String): String;
begin
  if strPath <> '' then
  begin
    Result := ExpandFileName(strPath);
  end
  else
  begin
    Result := '';
  end;
end;

function TAppItemList.RelativePath(strPath: String): String;
begin
  Result := ExtractRelativePath(fBasePath, strPath);
end;

procedure TAppItemList.CleanupSortOrder_UNUSED;
var
  I: Integer;
  intVal: Integer;
  intCur: Integer;
  intSrch: Integer;
  bolFound: Boolean;
  bolLower: Boolean;
  sl: TStringList;
begin
  sl := TStringList.Create;
  with TIniFile.Create(DataFile) do
  begin
    ReadSectionValues('SortOrder', sl);
    intSrch := 1;
    intCur := 1;
    bolLower := True;
    while bolLower do
    begin
      bolFound := False;
      bolLower := False;
      for I := 0 to sl.Count - 1 do
      begin
        intVal := StrToIntDef(sl.ValueFromIndex[I],0);
        if intVal > intSrch then
        begin
          bolLower := True;
        end else if intVal = intSrch then
        begin
          bolFound := True;
          sl[I] := Format('%s=%d', [ sl.Names[I], intCur ]);
        end;
      end;
      if bolFound then
      begin
        inc(intCur);
      end;
    end;
    EraseSection('SortOrder');
    For I := 0 to SL.Count - 1 do
    begin
      WriteString('SortOrder', SL.Names[I], SL.ValueFromIndex[I]);
    end;
    Free;
  end;
  SL.Free;
end;

end.
