unit CtgConfig;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, AppObjects, Dialogs, ExtDlgs;

type
  TdlgCtgConfig = class(TForm)
    btnSave: TButton;
    btnCancel: TButton;
    bvlFrame: TBevel;
    sbtnClosed: TSpeedButton;
    sbtnOpen: TSpeedButton;
    shpButtons: TShape;
    opndlgPickIcon: TOpenDialog;
    lbledName: TLabeledEdit;
    lbledHint: TLabeledEdit;
    procedure sbtnOpenClick(Sender: TObject);
    procedure sbtnClosedClick(Sender: TObject);
  private
    { Private declarations }
    MenuList: TAppItemList;
    CurrentItem: TAppItem;
    arrButtons: array of TSpeedButton;
  public
    { Public declarations }
    function Execute(MainMenuAppList: TAppItemList; intCurrentItem: Integer): Integer;
  end;

var
  dlgCtgConfig: TdlgCtgConfig;

implementation

uses main;

{$R *.dfm}

function TdlgCtgConfig.Execute(MainMenuAppList: TAppItemList; intCurrentItem: Integer): Integer;
var
  S: String;
begin
  Result := mrCancel;
  if MainMenuAppList.Apps[intCurrentItem].IsMenu = False then
    Exit;
  CurrentItem := MainMenuAppList.Apps[intCurrentItem];
  MenuList := TAppItemList.Create;
  MenuList.DefaultIconPath := MainMenuAppList.DefaultIconPath;
  MenuList.DefaultOpenIconPath := MainMenuAppList.DefaultOpenIconPath;
  MenuList.TransColor := clWhite;
  SetLength(arrButtons, 2);
  arrButtons[0] := sbtnOpen;
  arrButtons[1] := sbtnClosed;
  if CurrentItem.OpenIconHandle = 0 then
    S := frmMenu.strOpenChangeCaption
  else
    S := frmMenu.strOpenResetCaption;
  MenuList.AddCategory('open', S, '', True, 0, CurrentItem.IconPath, CurrentItem.OpenIconPath);
  if CurrentItem.IconHandle = 0 then
    S := frmMenu.strClosedChangeCaption
  else
    S := frmMenu.strClosedResetCaption;
  MenuList.AddCategory('closed', S, 'open', False, 0, CurrentItem.IconPath, CurrentItem.OpenIconPath);
  MenuList.DrawButtons;
  lbledName.Text := CurrentItem.Caption;
  lbledHint.Text := CurrentItem.Hint;
  // Set Captions
  dlgCtgConfig.Caption := frmMenu.strCategoryTitle + ' (' + CurrentItem.Name + ')';
  lbledName.EditLabel.Caption := frmMenu.strCategoryName;
  lbledHint.EditLabel.Caption := frmMenu.strCategoryHint;
  btnSave.Caption := frmMenu.strCategoryButtonSave;
  btnCancel.Caption := frmMenu.strCategoryButtonCancel;
  if ShowModal = mrOk then
  begin
    if lbledName.Text = '' then
      CurrentItem.Caption := CurrentItem.Name
    else
      CurrentItem.Caption := lbledName.Text;
    CurrentItem.Hint := lbledHint.Text;
    CurrentItem.IconPath := MenuList.Apps[1].IconPath;
    CurrentItem.OpenIconPath := MenuList.Apps[0].OpenIconPath;
    CurrentItem.AppList.Modified := True;
    Result := mrOK;
  end;
  MenuList.Free;
end;

procedure TdlgCtgConfig.sbtnClosedClick(Sender: TObject);
begin
  if MenuList.Apps[1].IconHandle = 0 then
  begin
    with opndlgPickIcon do
    begin
      if InitialDir = '' then
      begin
        if MenuList.Apps[1].IconHandle <> 0 then
          opndlgPickIcon.InitialDir := ExtractFilePath(MenuList.Apps[1].IconPath)
        else
          opndlgPickIcon.InitialDir := ExtractFilePath(MenuList.DefaultIconPath);
      end;
      if Execute then
      begin
        MenuList.Apps[1].IconPath := opndlgPickIcon.FileName;
        if MenuList.Apps[1].IconHandle <> 0 then
        begin
          MenuList.Apps[0].IconPath := opndlgPickIcon.FileName;
          MenuList.Apps[1].Caption := frmMenu.strClosedResetCaption;
        end;
      end;
    end;
  end else begin
    MenuList.Apps[1].IconPath := '';
    MenuList.Apps[1].Caption := frmMenu.strClosedChangeCaption;
    MenuList.Apps[0].IconPath := '';
  end;
  MenuList.DrawButtons;
end;

procedure TdlgCtgConfig.sbtnOpenClick(Sender: TObject);
begin
  if MenuList.Apps[0].OpenIconHandle = 0 then
  begin
    with opndlgPickIcon do
    begin
      if InitialDir = '' then
      begin
        if MenuList.Apps[0].IconHandle <> 0 then
          opndlgPickIcon.InitialDir := ExtractFilePath(MenuList.Apps[0].IconPath)
        else if MenuList.DefaultOpenIconPath <> '' then
          opndlgPickIcon.InitialDir := ExtractFilePath(MenuList.DefaultOpenIconPath)
        else
          opndlgPickIcon.InitialDir := ExtractFilePath(MenuList.DefaultIconPath);
      end;
      if Execute then
      begin
        if Copy(opndlgPickIcon.FileName, 0, 2) <> Copy(Application.ExeName,0,2) then
        begin
          ShowMessage(frmMenu.strCategoryDriveError);
        end else begin
          MenuList.Apps[0].OpenIconPath := opndlgPickIcon.FileName;
          if MenuList.Apps[0].OpenIconHandle <> 0 then
            MenuList.Apps[0].Caption := frmMenu.strOpenResetCaption;
        end;
      end;
    end;
  end else begin
    MenuList.Apps[0].OpenIconPath := '';
    MenuList.Apps[0].Caption := frmMenu.strOpenChangeCaption;
  end;
  MenuList.DrawButtons;
end;

end.
